-- --
-- 3D
-- --

-- Anlegen der Tabelle 

CREATE TABLE GeoDbLand3D (
  id     INTEGER,                          -- Identifikationsnummer
  name   VARCHAR(20),                      -- Objektname
  geo    SDO_GEOMETRY,                     -- Objektgeometrie
  CONSTRAINT pk_gdbland3D PRIMARY KEY(id)  -- Primrschlssel
);

-- Einfgen der 3D-Daten

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (1,'Flaggenmast',SDO_GEOMETRY(3001,NULL,SDO_POINT_TYPE(10,11,5),NULL,NULL));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (2,'Steg',SDO_GEOMETRY(3002,NULL,NULL,SDO_ELEM_INFO_ARRAY(1,2,1),SDO_ORDINATE_ARRAY(4,4,1, 5,7,2.5, 8,8,2)));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (3,'Baumreihe',SDO_GEOMETRY(3005,NULL,NULL,SDO_ELEM_INFO_ARRAY(1,1,3),
                      SDO_ORDINATE_ARRAY(10,19,5, 12,19,10, 14,19,7)));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (4,'Wall',SDO_GEOMETRY(3002,NULL,NULL,SDO_ELEM_INFO_ARRAY(1,2,2),
                 SDO_ORDINATE_ARRAY(1,15,3,2.414,15.586,3,3,17,3,3.586,18.414,3,5,19,3)));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (6,'Land',SDO_GEOMETRY(3007,NULL,NULL,SDO_ELEM_INFO_ARRAY(1,1003,1, 16,2003,1, 43,1003,1),
SDO_ORDINATE_ARRAY(1,1,1,19,1,1,19,20,1,1,20,1,1,1,1,4,5,1,3,10,1,7,14,1,13,15,1,17,12,1,16,8,1,13,5,1,9,3,1,4,5,1,7,7,2,12,7,2,13,11,2,10,13,2,7,10,2,7,7,2)));

COMMIT;

-- Metadaten bekannt geben

INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('GEODBLAND3D','GEO', SDO_DIM_ARRAY(SDO_DIM_ELEMENT('X',0,20,0.001),
                                           SDO_DIM_ELEMENT('Y',0,20,0.001),
                                           SDO_DIM_ELEMENT('Z',0,10,0.001)) ,
        NULL);
COMMIT;

-- Geometrieprfung

SELECT id, SDO_GEOM.VALIDATE_GEOMETRY_WITH_CONTEXT(geo,0.001) FROM GeoDbLand3D;

-- Geometrische Funktionen und Methoden

--- Abfrage Dimension:
SELECT name, g.geo.GET_DIMS() FROM GeoDbLand3D g;

-- minimal umgebendes Rechteck:
SELECT SDO_GEOM.SDO_MBR(geo) FROM GeoDbLand3D WHERE name = 'Steg';

-- minimal umgebendes Rechteck (Aggregat):
SELECT SDO_AGGR_MBR(geo) FROM GeoDbLand3D;

-- Lnge (auf 2D projiziert!):
SELECT name, SDO_GEOM.SDO_LENGTH(geo,0.001) FROM GeoDbLand3D WHERE name = 'Steg';

-- Abstand (dreidimensional berechnet):
SELECT SDO_GEOM.SDO_DISTANCE (geo, SDO_GEOMETRY(3001,NULL,SDO_POINT_TYPE(10,10,3),NULL,NULL), 0.001)
FROM GeoDbLand3D WHERE name = 'Flaggenmast';

-- berlappung (dreidimensional berechnet):
SELECT SDO_GEOM.RELATE(g1.geo,'ANYINTERACT',g2.geo,0.001)
FROM GeoDbLand3D g1, GeoDbLand3D g2
WHERE g1.name = 'Flaggenmast' AND g2.name = 'Land';

-- auf 2D projizieren;
SELECT SDO_CS.MAKE_2D(geo) AS geo2D
FROM GeoDbLand3D;

-- auf 3D erweitern
UPDATE GeoDbLand3D
SET geo = (SELECT SDO_CS.MAKE_3D(geo,5) FROM GeoDbLand WHERE id=1)
WHERE id = 1;

COMMIT;

-- Anfragen

-- R-Baum als Index anlegen
CREATE INDEX GeoDbLand3D_geo_ix
ON GeoDbLand3D(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX
PARAMETERS ('SDO_INDX_DIMS=3');

-- Rechteckanfrage
SELECT name
FROM GeoDbLand3D
WHERE SDO_FILTER (geo, SDO_GEOMETRY(3008,NULL,NULL,
                         SDO_ELEM_INFO_ARRAY(1,1007,3),
                         SDO_ORDINATE_ARRAY(4,7.5,2.4, 15,16,5))) = 'TRUE';

-- 3D-Flchen und Krper

-- Dach als 3D-Flche einfgen
INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (51,'Hausdach',
        SDO_GEOMETRY(3003,NULL,NULL,SDO_ELEM_INFO_ARRAY(1,1006,1, 1,1003,1, 16,2003,1, 31,1003,1),
          SDO_ORDINATE_ARRAY(9,9,6.5, 9,9.5,7, 12,9.5,7, 12,9,6.5, 9,9,6.5,
                             10,9.15,6.65, 11,9.15,6.65, 11,9.35,6.85, 10,9.35,6.85, 10,9.15,6.65,
                             9,10,6.5, 12,10,6.5, 12,9.5,7, 9,9.5,7, 9,10,6.5) ) );

-- Hauskrper als Quader mittels Eckpunktbeschreibung einfgen
INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (52,'Hauskrper',
        SDO_GEOMETRY(3008,NULL,NULL,SDO_ELEM_INFO_ARRAY(1,1007,3), SDO_ORDINATE_ARRAY(9,9,2, 12,10,6.5) ) );

COMMIT;

-- Prfen
SELECT name, SDO_GEOM.VALIDATE_GEOMETRY_WITH_CONTEXT(geo,0.001) FROM GeoDbLand3D WHERE name LIKE 'Haus%';
SELECT name, SDO_GEOM.SDO_MBR(geo) FROM GeoDbLand3D WHERE name LIKE 'Haus%';

-- Volumen berechnen
SELECT SDO_GEOM.SDO_VOLUME(geo,0.001) FROM GeoDbLand3D WHERE name = 'Hauskrper';


-- Test auf Schnitt: 
SELECT SDO_GEOM.RELATE (geo, 'ANYINTERACT',
                        SDO_GEOMETRY(3001,NULL,SDO_POINT_TYPE(9.5,9.25,6.75),NULL,NULL), 0.001)
FROM GeoDbLand3D
WHERE name = 'Hausdach';

SELECT SDO_GEOM.RELATE (geo, 'ANYINTERACT',
                        SDO_GEOMETRY(3001,NULL,SDO_POINT_TYPE(10.5,9.25,6.75),NULL,NULL), 0.001)
FROM GeoDbLand3D
WHERE name = 'Hausdach';

SELECT SDO_GEOM.RELATE (geo, 'ANYINTERACT',
                        SDO_GEOMETRY(3001,NULL,SDO_POINT_TYPE(10.5,9.5,4),NULL,NULL), 0.001)
FROM GeoDbLand3D
WHERE name = 'Hauskrper';


-- -------------
-- 3D-PUNKWOLKEN
-- -------------

-- Metadatentabelle anlegen:
CREATE TABLE PWMetaDaten (
  name    VARCHAR(50),                  -- Bezeichner
  pwinfo  SDO_PC,                       -- Metadaten
  CONSTRAINT pk_pwmt PRIMARY KEY(name)  -- Primrschlssel
);

-- Blocktabelle anlegen:
CREATE TABLE PWBlockDaten AS
  SELECT * FROM MDSYS.SDO_PC_BLK_TABLE;

-- Quelltabelle anlegen:
CREATE TABLE Quellpunkte (
  rid    VARCHAR(24),                  -- ID
  val_d1  NUMBER,                      -- x-Koordinatenwert
  val_d2  NUMBER,                      -- y-Koordinatenwert
  val_d3  NUMBER,                      -- z-Koordinatenwert
  CONSTRAINT pk_qpkt PRIMARY KEY(rid)  -- Primrschlssel
);

-- Nun Quellpunkte ber SQL*Loader laden

-- Quellpunkte visualisieren:
SELECT SDO_GEOMETRY(3001, NULL, SDO_POINT_TYPE(val_d1,val_d2,val_d3),NULL,NULL) FROM Quellpunkte;

-- berfhrung 
DECLARE
  pwinfo  SDO_PC;  -- das Metadaten-Objekt
BEGIN
  -- Metadaten-Objekt erzeugen:
  pwinfo := SDO_PC_PKG.INIT( 'PWMETADATEN', 'PWINFO', 'PWBLOCKDATEN', 'blk_capacity=50',
              SDO_GEOMETRY(3003, NULL, NULL, SDO_ELEM_INFO_ARRAY(1,1003,3),
                SDO_ORDINATE_ARRAY(-2559.079,-13712.342,2025.776, 
                                   -2091.243,-13478.560,7245.145)), 
	      1e-6, 3);
  -- Metadaten abspeichern:
  INSERT INTO PWMetaDaten VALUES ('Fassade',pwinfo);
  -- Daten berfhren:
  SDO_PC_PKG.CREATE_PC (pwinfo,'Quellpunkte');
END;
/

-- Zhlen:
SELECT COUNT(*), SUM(NUM_POINTS) FROM PWBlockDaten;

-- Umwandlung in 3D-MultiPoint-Geometrien:
SELECT SDO_PC_PKG.TO_GEOMETRY(points,num_points,3)
FROM PWBlockDaten
WHERE blk_id = 0;

-- Umwandlung in 2D-MultiPoint-Geometrien:
SELECT SDO_PC_PKG.TO_GEOMETRY(points,num_points,2)
FROM PWBlockDaten
WHERE blk_id = 0;

-- Quaderanfrage: Filterschritt
SELECT SDO_PC_PKG.TO_GEOMETRY(points,num_points,3)
FROM PWBlockDaten
WHERE SDO_FILTER (blk_extent, SDO_GEOMETRY(3003, NULL, NULL, SDO_ELEM_INFO_ARRAY(1,1003,3),
                SDO_ORDINATE_ARRAY(-2500,-13700,4000, 
                                   -2200,-13500,7000) ) ) = 'TRUE';

-- Quaderanfrage: Verfeinerungsschritt (Ergebnis seltsamerweise leer)
SELECT SDO_PC_PKG.TO_GEOMETRY(t.points,t.num_points,3)
FROM TABLE (
  SELECT SDO_PC_PKG.CLIP_PC (pwinfo,
            SDO_GEOMETRY(3003, NULL, NULL, SDO_ELEM_INFO_ARRAY(1,1003,3),
                SDO_ORDINATE_ARRAY(-2500,-13700,4000, 
                                   -2200,-13500,7000) ),
            NULL, NULL, NULL, NULL)
  FROM PWMetaDaten m
  WHERE name = 'Fassade') t;


-- ----
-- TINs
-- ----

DROP TABLE TINMetaDaten;
DROP TABLE TINBlockDaten;

-- Metadatentabelle anlegen:
CREATE TABLE TINMetaDaten (
  name     VARCHAR(50),                 -- Bezeichner
  tininfo  SDO_TIN,                     -- Metadaten
  CONSTRAINT pk_tinmt PRIMARY KEY(name) -- Primrschlssel
);

-- Blocktabelle anlegen:
CREATE TABLE TINBlockDaten AS
  SELECT * FROM MDSYS.SDO_TIN_BLK_TABLE;

-- Quellpunkte lschen:
DELETE FROM Quellpunkte;
COMMIT;

-- Nun Quellpunkte ber SQL*Loader laden

-- Quellpunkte visualisieren:
SELECT SDO_GEOMETRY(3001, NULL, SDO_POINT_TYPE(val_d1,val_d2,val_d3),NULL,NULL) FROM Quellpunkte;

SELECT SDO_GEOMETRY(2001, NULL, SDO_POINT_TYPE(val_d1,val_d2,null),NULL,NULL).Get_WKT() FROM Quellpunkte;

SELECT MIN(val_d1), MIN(val_d2), MAX(val_d1), MAX(val_d2) FROM Quellpunkte;

--- ggf. Aufrumen
DELETE FROM TINMetaDaten;
DELETE FROM TINBlockDaten;
DELETE FROM USER_SDO_GEOM_METADATA WHERE table_name = 'TINBLOCKDATEN';
COMMIT;

-- berfhrung 
DECLARE 
  tininfo     SDO_TIN;       -- das Metadaten-Objekt
  breaklines  SDO_GEOMETRY;  -- Bruchkanten
  stoplines   SDO_GEOMETRY;  -- Begrenzungslinien
  voidregions SDO_GEOMETRY;  -- Ebenen
BEGIN
  -- Metadaten-Objekt erzeugen:
  breaklines :=  NULL;      --  hier ggf. Bruchkanten spezifizieren 
  stoplines := NULL;        --  hier ggf. Begrenzungslinien spezifizieren
  voidregions := NULL;      --  hier ggf. Ebenen spezifizieren
  stoplines := SDO_GEOMETRY(2002,NULL,NULL, SDO_ELEM_INFO_ARRAY(1,2,1), SDO_ORDINATE_ARRAY(
		3468655,5934220, 3468655,5934620, 3469145,5934620, 3469145,5934220, 3468655,5934220));
  tininfo := SDO_TIN_PKG.INIT( 'TINMETADATEN', 'TININFO', 'TINBLOCKDATEN', 'blk_capacity=400',
               SDO_GEOMETRY(2003, NULL,NULL, SDO_ELEM_INFO_ARRAY(1,1003,3),
                 SDO_ORDINATE_ARRAY(3466614,5917982, 
                                    3466912,5918121)), 
	      0.01, 3, NULL, breaklines, stoplines, voidregions);
  -- Metadaten abspeichern:
  INSERT INTO TINMetaDaten VALUES ('TINDaten',tininfo);
  -- Daten berfhren:
  SDO_TIN_PKG.CREATE_TIN (tininfo,'Quellpunkte');
END;
/

-- Zhlen:
SELECT COUNT(*), SUM(NUM_TRIANGLES) FROM TINBlockDaten;

-- Umwandlung in 3D-GeometryCollection:
SELECT SDO_TIN_PKG.TO_GEOMETRY(points,triangles,num_points,num_triangles,2,3,NULL)
FROM TINBlockDaten;

-- Umwandlung in 2D-GeometryCollection:
SELECT SDO_CS.MAKE_2D(SDO_TIN_PKG.TO_GEOMETRY(points,triangles,num_points,num_triangles,2,3,NULL))
FROM TINBlockDaten;

-- Quaderanfrage
SELECT SDO_TIN_PKG.TO_GEOMETRY(t.points,t.triangles,t.num_points,t.num_triangles,2,3,NULL)
FROM TABLE (
  SELECT SDO_TIN_PKG.CLIP_TIN (tininfo,
            SDO_GEOMETRY(2003, NULL,NULL, SDO_ELEM_INFO_ARRAY(1,1003,3),
                SDO_ORDINATE_ARRAY(3466700,5918020, 
                                   3466800,5918080) ),
            NULL, NULL)
  FROM TINMetaDaten m
  WHERE name = 'TINDaten') t;
