-- ---------------------
-- GEOMETRISCHER VERBUND
-- ---------------------

--- Join mit nur einem Index 

-- Index von centrum lschen:
DROP INDEX Gemeinden_centrum_ix;

-- Filter:
SELECT /*+ ORDERED */ COUNT(*)
FROM Gemeinden g INNER JOIN Ortsnetze n
ON SDO_FILTER (n.gebiet, g.centrum) = 'TRUE';

-- Exakter Test:
SELECT /*+ ORDERED */ COUNT(*)
FROM Gemeinden g INNER JOIN Ortsnetze n
ON SDO_RELATE (n.gebiet, g.centrum, 'mask=CONTAINS') = 'TRUE';

-- ab V.10 auch mglich:
SELECT /*+ ORDERED */ COUNT(*)
FROM Gemeinden g INNER JOIN Ortsnetze n
ON SDO_CONTAINS (n.gebiet, g.centrum) = 'TRUE';

--- Join mit zwei Indexen

-- Index erzeugen:
CREATE INDEX Gemeinden_centrum_ix
ON Gemeinden(centrum)
INDEXTYPE IS MDSYS.SPATIAL_INDEX;

-- Filter:
SELECT /*+ ORDERED */ COUNT(*)
FROM Gemeinden g INNER JOIN Ortsnetze n
ON SDO_FILTER (n.gebiet, g.centrum) = 'TRUE';

-- Exakter Test:
SELECT /*+ ORDERED */ COUNT(*)
FROM Gemeinden g INNER JOIN Ortsnetze n
ON SDO_RELATE (n.gebiet, g.centrum, 'mask=CONTAINS') = 'TRUE';

-- ab V.10 mglich:
SELECT /*+ ORDERED */ COUNT(*)
FROM Gemeinden g INNER JOIN Ortsnetze n
ON SDO_CONTAINS (n.gebiet, g.centrum) = 'TRUE';

--- Join mit SDO_JOIN

-- SDO_JOIN:
SELECT SDO_JOIN('Gemeinden','centrum','Ortsnetze','gebiet','mask=INSIDE')
FROM Dual;

-- geht nicht fr ST_GEOMETRY:
SELECT SDO_JOIN('MMSpatialLand','geo','MMSpatialLand','geo','mask=INSIDE')
FROM Dual;

-- Ausgabe der RowID-Paare:
SELECT res.rowid1, res.rowid2
FROM TABLE(SDO_JOIN('Gemeinden','centrum','Ortsnetze','gebiet','mask=INSIDE')) res;

-- Ausgaben der Namen:
SELECT g.name AS Gemeindezentrum, n.name AS Ortsnetz
FROM TABLE(SDO_JOIN('Gemeinden','centrum','Ortsnetze','gebiet','mask=INSIDE')) res, Ortsnetze n, Gemeinden g
WHERE g.rowid = res.rowid1 AND n.rowid = res.rowid2;

-- Distance Join:
SELECT g.name AS Gemeindezentrum, n.name AS Ortsnetz
FROM TABLE(SDO_JOIN('Gemeinden','centrum','Ortsnetze','gebiet','distance=10 unit=KM')) res, Ortsnetze n, Gemeinden g
WHERE g.rowid = res.rowid1 AND n.rowid = res.rowid2;


-- ------------------------
-- NCHSTE-NACHBARN-ANFRAGE
-- ------------------------

--- mit Anfrageobjekt

SELECT n.name
FROM Ortsnetze n INNER JOIN Gemeinden g
ON SDO_NN (n.gebiet,g.centrum,'sdo_num_res=12') = 'TRUE'
WHERE g.name = 'Syke';

SELECT g1.name
FROM MMSpatialLand g1 INNER JOIN MMSpatialLand g2
ON SDO_NN (g1.geo,g2.geo,'sdo_num_res=4') = 'TRUE'
WHERE g2.name = 'Flaggenmast';

-- mit Anfrageobjekt und Entfernungsausgabe

SELECT n.name, SDO_NN_DISTANCE(1)
FROM Ortsnetze n INNER JOIN Gemeinden g
ON SDO_NN (n.gebiet,g.centrum,'sdo_num_res=12 unit=km',1) = 'TRUE'
WHERE g.name = 'Syke'
ORDER BY SDO_NN_DISTANCE(1);

-- nach Entfernung sortierte Ausgabe aller Ortsnetze

SELECT n.name
FROM Ortsnetze n INNER JOIN Gemeinden g
ON SDO_NN (n.gebiet,g.centrum) = 'TRUE'
WHERE g.name = 'Syke';

-- mit zustzlichen Bedingungen

-- unvollstndige NNQ:
SELECT n.name, SDO_NN_DISTANCE(1)
FROM Ortsnetze n INNER JOIN Gemeinden g
ON SDO_NN (n.gebiet,g.centrum,'sdo_num_res=5 unit=km',1) = 'TRUE'
WHERE g.name = 'Syke' AND n.name <> 'Syke'
ORDER BY SDO_NN_DISTANCE(1);

-- NNQ mit SDO_BATCH_SIZE:
SELECT n.name, SDO_NN_DISTANCE(1)
FROM Ortsnetze n INNER JOIN Gemeinden g
ON SDO_NN (n.gebiet,g.centrum,'sdo_batch_size=10 unit=km',1) = 'TRUE'
WHERE g.name = 'Syke' AND n.name <> 'Syke' AND ROWNUM <= 5
ORDER BY SDO_NN_DISTANCE(1);

-- NNQ mit SDO_BATCH_SIZE=0:
SELECT ROWNUM, n.name, SDO_NN_DISTANCE(1)
FROM Ortsnetze n INNER JOIN Gemeinden g
ON SDO_NN (n.gebiet,g.centrum,'sdo_batch_size=0 unit=km',1) = 'TRUE'
WHERE g.name = 'Syke' AND n.name <> 'Syke' AND ROWNUM <= 5
ORDER BY SDO_NN_DISTANCE(1);
